#!/usr/bin/perl -w

# Copyright 2021 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.

# Usage: perl gen-curve.pl 
#        perl gen-curve.pl p
#
# Print PGF/Tikz drawing for Mblock(p) interpreted as segment directions in
# the alternate paperfolding curve.

use 5.010;
use strict;
use warnings;
use File::Spec;

use FindBin;
use lib $FindBin::Bin;
use MyAppendMorphism 'Mblock','@s_to_dx','@s_to_dy';
use MyLineSegments;
use MyPrintwrap ':all';
$|=1;


my $max_p = 8;
if (@ARGV) { $max_p = shift @ARGV; }

print "    % generated by tools/$FindBin::Script $max_p\n";
print "    \\newcommand\\MyMaxP{$max_p}\n";

my $segs = MyLineSegments->new;

foreach my $p (0 .. $max_p) {
  my @Mblock = Mblock($p);
  my $x = 0;
  my $y = 0;
  foreach my $i (0 .. $#Mblock) {
    my $s = $Mblock[$i];
    my $dx = $s_to_dx[$s];
    my $dy = $s_to_dy[$s];
    my $x2 = $x + $dx;
    my $y2 = $y + $dy;
    $segs->add($x,$y, $x2,$y2);
    ($x,$y) = ($x2,$y2);
  }
}

$Printwrap = 4;
printwrap("\\draw ");
$Printwrap += 2;
$segs->printwrap;
printwrap(";\n");

print "    % end generated\n";
exit 0;
